<template>
  <Dialog v-model="dialogVisible" :title="dialogTitle" width="65%">
    <el-form
      ref="formRef"
      v-loading="formLoading"
      :model="dataForm"
      :rules="formRules"
      label-width="120px"
    >
      <el-row>
        <el-col :span="12">
          <el-form-item label="调度模型" prop="scheduleModelId">
            <el-select v-model="dataForm.scheduleModelId" clearable placeholder="请选择调度模型">
              <el-option
                v-for="item in scheduleModelList"
                :key="item.id"
                :label="item.modelName"
                :value="item.id"
              />
            </el-select>
          </el-form-item>
        </el-col>
        <el-col :span="12">
          <el-form-item label="状态" prop="status">
            <el-select
              v-model="dataForm.status"
              placeholder="请选择"
              clearable
              style="width: 100%">
              <el-option
                v-for="dict in getIntDictOptions(DICT_TYPE.MODEL_STATUS)"
                :key="dict.value"
                :label="dict.label"
                :value="dict.value"
              />
            </el-select>
          </el-form-item>
        </el-col>
      </el-row>
      <el-row>
        <el-col :span="24">
          <el-form-item label="备注" prop="remark">
            <el-input v-model="dataForm.remark" placeholder="请输入备注"/>
          </el-form-item>
        </el-col>
      </el-row>
      <el-divider content-position="left">配置</el-divider>
      <el-button
        @click="addAdjustConfigDet()"
        type="primary"
        size="small">
        添加
      </el-button>
      <el-table
        :data="dataForm.adjustConfigDetList"
        border
        style="width: 100%; margin-top: 5px;">
        <el-table-column prop="sort" label="执行顺序" align="center" width="90px"/>
        <el-table-column label="预测项类型" align="center" width="220px">
          <template #default="scope">
            <el-select v-model="scope.row.itemTypeId" placeholder="请选择" clearable @change="changeitemTypeId(scope.row)">
              <el-option
                v-for="item in itemTypeList"
                :key="item.id"
                :label="item.itemtypename"
                :value="item.id"/>
            </el-select>
          </template>
        </el-table-column>
        <el-table-column label="预测项名" align="center" width="300px">
          <template #default="scope">
            <el-select
              v-model="scope.row.predictItemId"
              placeholder="请选择"
              clearable>
              <el-option
                v-for="item in predictItemList"
                :key="item.id"
                :label="item.itemname"
                :value="item.id"/>
            </el-select>
          </template>
        </el-table-column>
        <el-table-column label="输出KEY" align="center">
          <template #default="scope">
            <el-input v-model="scope.row.outKey" placeholder="输出KEY"/>
          </template>
        </el-table-column>
        <el-table-column label="输出名称" align="center">
          <template #default="scope">
            <el-input v-model="scope.row.outName" placeholder="输出名称"/>
          </template>
        </el-table-column>
        <el-table-column label="参数排序" align="center" width="105px">
          <template #default="scope">
            <el-input v-model="scope.row.modelParamOrder" placeholder="参数排序"/>
          </template>
        </el-table-column>
        <el-table-column label="输入排序" align="center" width="105px">
          <template #default="scope">
            <el-input v-model="scope.row.modelParamPortOrder" placeholder="输入排序"/>
          </template>
        </el-table-column>
        <el-table-column prop="" label="操作" align="center" width="100px">
          <template #default="scope">
            <el-button
              @click="deleteAdjustConfigDet(scope.$index, dataForm.adjustConfigDetList)"
              type="text"
              size="small">
              删除
            </el-button>
          </template>
        </el-table-column>
      </el-table>
    </el-form>
    <template #footer>
      <el-button :disabled="formLoading" type="primary" @click="submitForm">确 定</el-button>
      <el-button @click="dialogVisible = false">取 消</el-button>
    </template>
  </Dialog>
</template>
<script lang="ts" setup>
import * as AdjustConfigApi from '@/api/model/sche/adjust'
import * as MmItemType from "@/api/model/pre/type";
import * as MmPredictItem from "@/api/model/pre/item";
import * as ScheduleModelApi from "@/api/model/sche/model";
import {DICT_TYPE, getIntDictOptions} from "@/utils/dict";

defineOptions({name: 'AdjustConfigForm'})

const {t} = useI18n() // 国际化
const message = useMessage() // 消息弹窗
const dialogVisible = ref(false) // 弹窗的是否展示
const dialogTitle = ref('') // 弹窗的标题
const formLoading = ref(false) // 表单的加载中：1）修改时的数据加载；2）提交的按钮禁用
const formType = ref('') // 表单的类型：create - 新增；update - 修改
const itemTypeList = ref([])
const predictItemList = ref([])
const scheduleModelList = ref([] as ScheduleModelApi.ScheduleModelVO[])
const modelparamListMap = ref({})
const dataForm = ref({
  id: undefined,
  scheduleModelId: undefined,
  status: undefined,
  remark: undefined,
  adjustConfigDetList: [],
})
const formRules = reactive({
  scheduleModelId: [{required: true, message: '调度模型不能为空', trigger: 'blur'}],
  status: [{required: true, message: '状态不能为空', trigger: 'blur'}],
})
const queryParams = reactive({
  itemtypeid: undefined,
})
const formRef = ref() // 表单 Ref

/** 打开弹窗 */
const open = async (type: string, id?: number) => {
  dialogVisible.value = true
  dialogTitle.value = t('action.' + type)
  formType.value = type
  resetForm()
  // 修改时，设置数据
  if (id) {
    formLoading.value = true
    try {
      dataForm.value = await AdjustConfigApi.getAdjustConfig(id)
    } finally {
      formLoading.value = false
    }
  }
  // 加载调度模型列表
  scheduleModelList.value = await ScheduleModelApi.getScheduleModelList()
  // 加载参数列表
  modelparamListMap.value = await ScheduleModelApi.getModelParamList()
  // 获取预测项类型列表
  itemTypeList.value = await MmItemType.getItemTypeList()

  predictItemList.value = await MmPredictItem.getMmPredictItemList(queryParams)
}

defineExpose({open}) // 提供 open 方法，用于打开弹窗

/** 提交表单 */
const emit = defineEmits(['success']) // 定义 success 事件，用于操作成功后的回调
const submitForm = async () => {
  // 校验表单
  if (!formRef) return
  const valid = await formRef.value.validate()
  if (!valid) return
  // 提交请求
  formLoading.value = true
  try {
    const data = dataForm.value as unknown as AdjustConfigApi.AdjustConfigVO
    if (formType.value === 'create') {
      await AdjustConfigApi.createAdjustConfig(data)
      message.success(t('common.createSuccess'))
    } else {
      await AdjustConfigApi.updateAdjustConfig(data)
      message.success(t('common.updateSuccess'))
    }
    dialogVisible.value = false
    // 发送操作成功的事件
    emit('success')
  } finally {
    formLoading.value = false
  }
}

function changeitemTypeId(row) {
  row.predictItemId = ''
  getInfo(row.itemTypeId)
}

const getInfo = async (itemTypeId) => {
  queryParams.itemtypeid = itemTypeId
  predictItemList.value = await MmPredictItem.getMmPredictItemList(queryParams)
}

function addAdjustConfigDet() {
  if (!dataForm.value.adjustConfigDetList) {
    dataForm.value.adjustConfigDetList = []
  }
  dataForm.value.adjustConfigDetList.push({})
  orderDet(dataForm.value.adjustConfigDetList)
}

function deleteAdjustConfigDet(index: string, rows) {
  if (!rows || rows.length === 1) {
    message.error('不能全部删除!')
    return
  }
  rows.splice(index, 1)
  orderDet(rows)
}

function orderDet(list) {
  list.sort((a, b) => a.sort - b.sort);
  let sort = 1
  list.forEach(function (value) {
    value.sort = sort
    sort++
  })
}

/** 重置表单 */
const resetForm = () => {
  dataForm.value = {
    id: undefined,
    scheduleModelId: undefined,
    status: undefined,
    remark: undefined,
    adjustConfigDetList: [],
  }
  formRef.value?.resetFields()

}
</script>
